@@ -92,9 +92,17 @@ def gmshVersion(communicator=parallelComm):
 
 def _gmshVersion(communicator=parallelComm):
     version = gmshVersion(communicator) or "0.0"
-    return StrictVersion(version)
+    try:
+        version = StrictVersion(version)
+    except ValueError:
+        # gmsh returns the version string in stderr,
+        # which means it's often unparsable due to irrelevant warnings
+        # assume it's OK and move on
+        version = StrictVersion("3.0")
 
-def openMSHFile(name, dimensions=None, coordDimensions=None, communicator=parallelComm, order=1, mode='r', background=None):
+    return version
+
+def openMSHFile(name, dimensions=None, coordDimensions=None, communicator=parallelComm, overlap=1, mode='r', background=None):
     """Open a Gmsh `MSH` file
 
     Parameters
@@ -105,8 +113,12 @@ def openMSHFile(name, dimensions=None, coordDimensions=None, communicator=parall
         Dimension of mesh
     coordDimensions : int
         Dimension of shapes
-    order
-        ???
+    overlap : int
+        The number of overlapping cells for parallel
+        simulations. Generally 1 is adequate. Higher order equations or
+        discretizations require more. If `overlap` is greater than one,
+        communication reverts to serial, as Gmsh only provides one layer
+        of ghost cells.
     mode : str
         Beginning with `r` for reading and `w` for writing.
         The file will be created if it doesn't exist when opened for writing;
@@ -116,7 +128,7 @@ def openMSHFile(name, dimensions=None, coordDimensions=None, communicator=parall
         Specifies the desired characteristic lengths of the mesh cells
     """
 
-    if order > 1:
+    if overlap > 1:
         communicator = serialComm
 
     # Enforce gmsh version to be either >= 2 or 2.5, based on Nproc.
@@ -164,8 +176,8 @@ def openMSHFile(name, dimensions=None, coordDimensions=None, communicator=parall
             gmshFlags = ["-%d" % dimensions, "-nopopup"]
 
             if communicator.Nproc > 1:
-                if version < StrictVersion("2.5"):
-                    warnstr = "Cannot partition with Gmsh version < 2.5. " \
+                if not (StrictVersion("2.5") < version <= StrictVersion("4.0")):
+                    warnstr = "Cannot partition with Gmsh version < 2.5 or >= 4.0. " \
                                + "Reverting to serial."
                     warnings.warn(warnstr, RuntimeWarning, stacklevel=2)
                     communicator = serialComm
@@ -176,6 +188,11 @@ def openMSHFile(name, dimensions=None, coordDimensions=None, communicator=parall
                         raise ValueError("'dimensions' must be specified to generate a mesh from a geometry script")
                 else: # gmsh version is adequate for partitioning
                     gmshFlags += ["-part", "%d" % communicator.Nproc]
+                    if version >= StrictVersion("4.0"):
+                        # Gmsh 4.x needs to be told to generate ghost cells
+                        # Unfortunately, the ghosts are broken
+                        # https://gitlab.onelab.info/gmsh/gmsh/issues/733
+                        gmshFlags += ["-part_ghosts"]
 
             gmshFlags += ["-format", "msh2"]
 
@@ -1514,23 +1531,24 @@ class Gmsh2D(Mesh2D):
 
     >>> from fipy import CellVariable, numerix
 
-    >>> std = []
+    >>> error = []
     >>> bkg = None
     >>> from builtins import range
     >>> for refine in range(4):
     ...     square = Gmsh2D(geo, background=bkg) # doctest: +GMSH
     ...     x, y = square.cellCenters # doctest: +GMSH
     ...     bkg = CellVariable(mesh=square, value=abs(x / 4) + 0.01) # doctest: +GMSH
-    ...     std.append((numerix.sqrt(2 * square.cellVolumes) / bkg).std()) # doctest: +GMSH
+    ...     error.append(((2 * numerix.sqrt(square.cellVolumes) / bkg - 1)**2).cellVolumeAverage) # doctest: +GMSH
 
-    Check that the mesh is monotonically approaching the desired density
+    Check that the mesh is (semi)monotonically approaching the desired density
+    (the first step may increase, depending on the number of partitions)
 
-    >>> print(numerix.greater(std[:-1], std[1:]).all()) # doctest: +GMSH
+    >>> print(numerix.greater(error[:-1], error[1:]).all()) # doctest: +GMSH
     True
 
     and that the final density is close enough to the desired density
 
-    >>> print(std[-1] < 0.2) # doctest: +GMSH
+    >>> print(error[-1] < 0.02) # doctest: +GMSH
     True
 
     The initial mesh doesn't have to be from Gmsh
@@ -1557,8 +1575,12 @@ class Gmsh2D(Mesh2D):
         Gmsh geometry (`.geo`) file, or (iii) a Gmsh geometry script
     coordDimensions : int
         Dimension of shapes
-    order
-        ???
+    overlap : int
+        The number of overlapping cells for parallel
+        simulations. Generally 1 is adequate. Higher order equations or
+        discretizations require more. If `overlap` is greater than one,
+        communication reverts to serial, as Gmsh only provides one layer
+        of ghost cells.
     background : ~fipy.variables.cellVariable.CellVariable
         Specifies the desired characteristic lengths of the mesh cells
     """
@@ -1567,17 +1589,21 @@ class Gmsh2D(Mesh2D):
                  arg,
                  coordDimensions=2,
                  communicator=parallelComm,
-                 order=1,
+                 overlap=1,
                  background=None):
 
         self.mshFile = openMSHFile(arg,
                                    dimensions=2,
                                    coordDimensions=coordDimensions,
                                    communicator=communicator,
-                                   order=order,
+                                   overlap=overlap,
                                    mode='r',
                                    background=background)
 
+        # openMSHFile may have "downgraded" the communicator
+        # if, e.g., too many overlaps were requested
+        communicator = self.mshFile.communicator
+
         (verts,
          faces,
          cells,
@@ -1856,17 +1882,21 @@ class Gmsh2DIn3DSpace(Gmsh2D):
         Gmsh geometry (`.geo`) file, or (iii) a Gmsh geometry script
     coordDimensions : int
         Dimension of shapes
-    order
-        ???
+    overlap : int
+        The number of overlapping cells for parallel
+        simulations. Generally 1 is adequate. Higher order equations or
+        discretizations require more. If `overlap` is greater than one,
+        communication reverts to serial, as Gmsh only provides one layer
+        of ghost cells.
     background : ~fipy.variables.cellVariable.CellVariable
         Specifies the desired characteristic lengths of the mesh cells
     """
-    def __init__(self, arg, communicator=parallelComm, order=1, background=None):
+    def __init__(self, arg, communicator=parallelComm, overlap=1, background=None):
         Gmsh2D.__init__(self,
                         arg,
                         coordDimensions=3,
                         communicator=communicator,
-                        order=order,
+                        overlap=overlap,
                         background=background)
 
     def _test(self):
@@ -1934,19 +1964,27 @@ class Gmsh3D(Mesh):
     arg : str
         (i) the path to an `MSH` file, (ii) a path to a
         Gmsh geometry (`.geo`) file, or (iii) a Gmsh geometry script
-    order
-        ???
+    overlap : int
+        The number of overlapping cells for parallel
+        simulations. Generally 1 is adequate. Higher order equations or
+        discretizations require more. If `overlap` is greater than one,
+        communication reverts to serial, as Gmsh only provides one layer
+        of ghost cells.
     background : ~fipy.variables.cellVariable.CellVariable
         Specifies the desired characteristic lengths of the mesh cells
     """
-    def __init__(self, arg, communicator=parallelComm, order=1, background=None):
+    def __init__(self, arg, communicator=parallelComm, overlap=1, background=None):
         self.mshFile  = openMSHFile(arg,
                                     dimensions=3,
                                     communicator=communicator,
-                                    order=order,
+                                    overlap=overlap,
                                     mode='r',
                                     background=background)
 
+        # openMSHFile may have "downgraded" the communicator
+        # if, e.g., too many overlaps were requested
+        communicator = self.mshFile.communicator
+
         (verts,
          faces,
          cells,
@@ -2168,7 +2206,7 @@ class GmshGrid2D(Gmsh2D):
     """Should serve as a drop-in replacement for `Grid2D`
     """
     def __init__(self, dx=1., dy=1., nx=1, ny=None,
-                 coordDimensions=2, communicator=parallelComm, order=1):
+                 coordDimensions=2, communicator=parallelComm, overlap=1):
         self.dx = dx
         self.dy = dy or dx
         self.nx = nx
@@ -2176,7 +2214,7 @@ class GmshGrid2D(Gmsh2D):
 
         arg = self._makeGridGeo(self.dx, self.dy, self.nx, self.ny)
 
-        Gmsh2D.__init__(self, arg, coordDimensions, communicator, order, background=None)
+        Gmsh2D.__init__(self, arg, coordDimensions, communicator, overlap, background=None)
 
     @property
     def _meshSpacing(self):
@@ -2225,7 +2263,7 @@ class GmshGrid2D(Gmsh2D):
         >>> yogmsh.cellCenters.value.size == yogrid.cellCenters.value.size # doctest: +GMSH
         True
 
-        >>> mesh = GmshGrid2D(nx=2, ny=2) # doctest: +GMSH
+        >>> mesh = GmshGrid2D(nx=2, ny=2, communicator=serialComm) # doctest: +GMSH
 
         >>> mesh.numberOfCells == 4 # doctest: +GMSH
         True
@@ -2238,7 +2276,7 @@ class GmshGrid3D(Gmsh3D):
     """Should serve as a drop-in replacement for `Grid3D`
     """
     def __init__(self, dx=1., dy=1., dz=1., nx=1, ny=None, nz=None,
-                 communicator=parallelComm, order=1):
+                 communicator=parallelComm, overlap=1):
         self.dx = dx
         self.dy = dy or dx
         self.dz = dz or dx
@@ -2250,7 +2288,7 @@ class GmshGrid3D(Gmsh3D):
         arg = self._makeGridGeo(self.dx, self.dy, self.dz,
                                 self.nx, self.ny, self.nz)
 
-        Gmsh3D.__init__(self, arg, communicator=communicator, order=order)
+        Gmsh3D.__init__(self, arg, communicator=communicator, overlap=overlap)
 
     @property
     def _meshSpacing(self):
@@ -2310,7 +2348,8 @@ class GmshGrid3D(Gmsh3D):
         >>> numerix.allclose(yogmsh._faceAreas, yogrid._faceAreas) # doctest: +GMSH
         True
 
-        >>> mesh = GmshGrid3D(nx=2, ny=2, nz=2) # doctest: +GMSH
+        >>> mesh = GmshGrid3D(nx=2, ny=2, nz=2,
+        ...                   communicator=serialComm) # doctest: +GMSH
 
         >>> ccs = [[ 0.5,  0.5,  0.5,  0.5,  1.5,  1.5,  1.5,  1.5],
         ...    [ 0.5,  0.5,  1.5,  1.5,  0.5,  0.5,  1.5,  1.5],
